#!/usr/bin/env python3

from obt import path, wget
import os

url=["http://www.tweakoz.com/resources/audio/Feb142023.mp3"]
md5="d6ade30fdc8f9c80a7c846c983b4877a"
#url = ["https://www.kozco.com/tech/c304-2.wav"]
#md5 = "df22f8d0f920eb8cdd52a7bb84a1dc03"
#url = ["https://www2.cs.uic.edu/~i101/SoundFiles/ImperialMarch60.wav"]
#md5 = "4f4ce82317b15ee9cebc9b75c90509c0"

wav_path = wget.wget(urls=url,
                     output_name=path.temp()/"test.mp3",
                     md5val=md5)

assert(wav_path!=None)  

os.system("ffmpeg -i %s -f wav - | pw-play -" % wav_path)



from obt import dep, pip, path, dep, pathtools, command, log, env
Command = command.Command

###############################################################################
VERSION = "1.28"
###############################################################################

class renderdoc(dep.Provider):

  def __init__(self): ############################################
    super().__init__("renderdoc")
    rdoc_dir = path.Path("/opt/renderdoc_%s"%VERSION)
    self.rdoc_dir = rdoc_dir
    self.rdoc_bin = rdoc_dir/"bin"/"qrenderdoc"
    self._oslist = ["Linux"]
    self._archlist = ["x86_64"]

  ########

  def build(self): ############################################################
      return 0

  ########

  def env_init(self):
    log.marker("registering renderdoc(%s) SDK"%VERSION)
    env.append("PATH",self.rdoc_dir/"bin")
    env.set("RENDERDOC_DIR",self.rdoc_dir)
    env.set("RENDERDOC_VER",VERSION)

  ########

  def on_build_shell(self):
    pathtools.mkdir(self.build_dest,clean=False)
    return command.subshell( directory=self.build_dest,
                             prompt = "RENDERDOC",
                             environment = dict() )

  ########

  def areRequiredSourceFilesPresent(self):
    return self.rdoc_bin.exists()

  def areRequiredBinaryFilesPresent(self):
    return None



from obt._dep_build import BaseBuilder
from obt._dep_impl import require
from obt import pathtools, path, _globals
from obt import make
from obt.command import Command
import obt.host
from collections.abc import Callable

###############################################################################

class AutoConfBuilder(BaseBuilder):
  ###########################################
  def __init__(self,name):
    super().__init__(name)
    ##################################
    self._needaclocal = False
    self._needsautogendotsh = False
    self._parallelism=1.0
    self._options = list()
    self._envvars = dict()
    if _globals.tryBoolOption("serial"):
      self._parallelism=0.0
  ###########################################
  def requires(self,deplist):
    self._deps += deplist
  ###########################################
  def setEnvVar(self,key,value):
    self._envvars[key] = value
  ###########################################
  def setConfVar(self,key,value):
    self._confvar[key] = value
  ###########################################
  def setOption(self,opt):
    self._options += [opt]
  ###########################################
  def setConfVars(self,othdict):
    for k in othdict:
      self._confvar[k] = othdict[k]
  ###########################################
  def build(self,srcdir,blddir,wrkdir,incremental=False):
    ok2build = require(self._deps)
    if not ok2build:
      return False
    retc = 0
    pathtools.mkdir(blddir)
    if incremental:
      os.chdir(blddir)
    else:

      if self._needaclocal:
        pathtools.chdir(srcdir)
        retc = Command(["aclocal"]).exec()
        if retc!=0:
          print(deco.red("Error running aclocal<%d>"%retc))
          return False
      if self._needsautogendotsh:
        pathtools.chdir(srcdir)
        retc = Command(["./autogen.sh"]).exec()
        if retc == 0:
           make.exec("distclean")
        else:
          print(deco.red("Error running autogen.sh<%d>"%retc))
          return False

      pathtools.mkdir(blddir,clean=True)
      pathtools.chdir(blddir)

      retc = Command([srcdir/"configure",
                      '--prefix=%s'%path.prefix()
                     ]+self._options,
                     environment=self._envvars
                     ).exec()
      if retc==0:
        make.exec("all")
        retc = make.exec("install",parallelism=0.0)
    print("retc<%d>"%int(retc))
    return retc==0
  ###########################################
  def install(self,blddir):
    pathtools.chdir(blddir)
    return (make.exec("install",parallelism=0.0)==0)
  ###########################################




from obt import dep, path, command, docker, wget, pathtools, sdk, conan, target
from obt.deco import Deco
import obt.module
import time, re, socket, os, sys
from pathlib import Path
deco = obt.deco.Deco()

this_path = os.path.realpath(__file__)
this_dir = Path(os.path.dirname(this_path))
PREFIX = path.subspace_root()/"ios"

###############################################################################

cmake_tc = """
set(IOS_SDK_PATH $ENV{IOS_SDK_DIR})
set(CMAKE_SYSTEM_NAME iOS)

# Specify the architectures to build for (e.g., arm64 for actual devices)
set(CMAKE_OSX_ARCHITECTURES arm64)

# Specify the minimum iOS deployment target
set(CMAKE_OSX_DEPLOYMENT_TARGET $ENV{IOS_SDK_VER})

# Specify the path to the compiler
set(CMAKE_C_COMPILER $ENV{IOS_CLANG_PATH})
set(CMAKE_CXX_COMPILER $ENV{IOS_CLANGPP_PATH})
set(CMAKE_INSTALL_PREFIX $ENV{OBT_SUBSPACE_BUILD_DIR})
"""

###############################################################################

conan_host_profile = """
[settings]
os=iOS
os.version=17.0
arch=armv8
compiler=apple-clang
compiler.version=15.0
compiler.libcxx=libc++
build_type=Release
os.sdk=iphoneos
"""

conan_build_profile = """
[settings]
os=Macos
arch=armv8  
compiler=apple-clang
compiler.version=15.0
compiler.libcxx=libc++
build_type=Release
"""


###############################################################################

class _iossubspace_private:

  _instance = None

  def __new__(cls):
    if cls._instance is None:
      cls._instance = super().__new__(cls)
      cls._instance._initialize()
    return cls._instance

  def _initialize(self):
    pathtools.ensureDirectoryExists(PREFIX)
    pathtools.ensureDirectoryExists(PREFIX/"manifests")
    pathtools.ensureDirectoryExists(PREFIX/"builds")
    pathtools.ensureDirectoryExists(PREFIX/"include")
    pathtools.ensureDirectoryExists(PREFIX/"lib")
    pathtools.ensureDirectoryExists(PREFIX/"bin")
    pathtools.ensureDirectoryExists(PREFIX/"conan")
    host_profile_path = PREFIX/"ios.host.profile"
    build_profile_path = PREFIX/"ios.build.profile"
    if not host_profile_path.exists():
      with open(host_profile_path,"w") as f:
        f.write(conan_host_profile)
    if not build_profile_path.exists():
      with open(build_profile_path,"w") as f:
        f.write(conan_build_profile)

###############################################################################

class subspaceinfo:
    ###############################################
    def __init__(self):
      super().__init__()
      self._name = "ios"
      self._subsrc = this_dir
      self._prefix = PREFIX
      self._manifest_path = path.manifests()/self._name
      self._target = target.descriptor("aarch64","ios")
    ###############################################
    # build the docker image
    ###############################################
    def build(self,build_args,do_wipe=False):
      self._private = _iossubspace_private()
      os.chdir(self._prefix)
      #####################
      # generate cmake toolchain
      #####################
      tc_output = self._prefix/"ios.toolchain.cmake"
      with open(tc_output,"w") as f:
        f.write(cmake_tc)     
    ###############################################
    def _gen_sysprompt(self):
      return "ios"
    ###############################################
    def _gen_environment(self):
      
      if "DEVELOPMENT_TEAM" not in os.environ:
        print(deco.err("DEVELOPMENT_TEAM not set"))
        assert(False)
      
      TEMP_PATH = path.temp()
      IOS_SDK = sdk.descriptor("aarch64","ios")
      SDK_DIR = IOS_SDK._sdkdir
      SDK_VER = IOS_SDK._sdkver
      
      orig_path = os.environ["PATH"]
      the_environ = {
        "OBT_SUBSPACE_BUILD_DIR": self._prefix/"builds",
        "OBT_SUBSPACE_LIB_DIR": self._prefix/"lib",
        "OBT_SUBSPACE_DIR": self._prefix,
        "OBT_SUBSPACE_BIN_DIR": self._prefix/"bin",
        "IOS_PREFIX": self._prefix,
        "IOS_SDK_DIR": SDK_DIR,
        "IOS_SDK_VER": SDK_VER,
        "IOS_CLANG_PATH": IOS_SDK._clang_path,
        "IOS_CLANGPP_PATH": IOS_SDK._clangpp_path,
        "OBT_SUBSPACE": "ios",
        "CMAKE_TOOLCHAIN_FILE": self._prefix/"ios.toolchain.cmake",
        "OBT_SUBSPACE_PROMPT": self._gen_sysprompt(),
        "OBT_TARGET": "aarch64-ios",
        "PATH": orig_path+":"+str(this_dir/"bin"),
      }       
      the_environ.update(conan.environment()) 
      return the_environ
    ###############################################
    # launch conda subspace shell
    ###############################################
    def shell(self,working_dir=None,container=None):
      self._private = _iossubspace_private()
      sys.path.append(os.environ["OBT_BIN_PUB_DIR"])
      from _obt_config import configFromEnvironment
      import obt._envutils 
      obt_config = configFromEnvironment()
      sub_env = obt._envutils.EnvSetup(obt_config)
      #############################
      BASHRC = sub_env.genBashRc()
      BASHRC += "\n\n"
      BASHRC += "%s --stack\n"%(self._prefix/"bin"/"activate")
      fname = None
      rval = 0
      environ = self._gen_environment()
      import tempfile
      bash_command = ["bash"]
      with tempfile.NamedTemporaryFile(dir=path.temp(),mode="w",delete=False) as tempf:
        fname = tempf.name
        tempf.write(BASHRC)
        tempf.close()
        print(fname)
        bash_command += ["--rcfile",fname, "-i"]
        rval = command.run(bash_command,working_dir=working_dir,environment=environ,do_log=True)
      #############################
      return rval

    ###############################################
    # launch docker container
    #  print out connection info
    ###############################################
    def launch(self,launch_args):
      self._private = _iossubspace_private()
      print("launch ios")
    ###############################################
    # information dictionary
    ###############################################
    def info(self):
      return {
        "name": "ios",
        "prefix": self._prefix,
      }
    ###############################################




###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt import dep, path, osrelease, host

###############################################################################

class depthaicore(dep.StdProvider):
  name = "depthaicore"
  def __init__(self):
    super().__init__(depthaicore.name)
    self.declareDep("cmake")
    self._builder = self.createBuilder(dep.CMakeBuilder)
    self._builder._cmakeenv = {
      "BUILD_SHARED_LIBS": "ON",
      "DEPTHAI_BUILD_TESTS": "ON",
    	"DEPTHAI_BUILD_EXAMPLES": "OFF"
    }
    desc = osrelease.descriptor()
    if desc.version_id == "23.10" and host.IsLinux:
      self._builder.setCmVar("CMAKE_CXX_COMPILER","g++-11")
      self._builder.setCmVar("CMAKE_C_COMPILER","gcc-11")

	# echo 'SUBSYSTEM=="usb", ATTRS{idVendor}=="03e7", MODE="0666"' | sudo tee /etc/udev/rules.d/80-movidius.rules
  # sudo udevadm control --reload-rules && sudo udevadm trigger
  
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=depthaicore.name,
                             repospec="tweakoz/depthai-core",
                             revision="main",
                             recursive=True)

  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libxxxx.so").exists()



import obt.xcode 
import obt.command 
import obt.env
import obt.path 
import os
import subprocess

class _xrossdk_private:
  def __init__(self):
    self._xcodesdkstr = obt.command.capture([
      "xcodebuild",
      "-version",
      "-sdk"])
    self.has_xrosdk_1 = self._xcodesdkstr.find("(xros1.0)")>0
    print("has_xrosdk_1<%s>"%self.has_xrosdk_1)
    if self.has_xrosdk_1:
      self._xrosdkver = "1.0"
      self._xrosdkstr = obt.command.capture([
        "xcodebuild",
        "-version",
        "-sdk", "xros1.0",
        "Path"],do_log=False).splitlines()[0].strip("\"")
    try:
      # Use xcrun to find the path to clang for the current SDK
      clang_path = subprocess.check_output([
        "xcrun",
        "-find",
        "clang"
      ], text=True).strip().strip("\"")
      clangpp_path = subprocess.check_output([
        "xcrun",
        "-find",
        "clang++"
      ], text=True).strip().strip("\"")

      # Set environment variable for the compiler path
      self._clang_path = clang_path
      self._clangpp_path = clangpp_path

    except subprocess.CalledProcessError as e:
      print(f"Error executing command: {e}")
    except Exception as e:
      print(f"An unexpected error occurred: {e}")


class sdkinfo:
  #################################################
  # env_init invoked once at container startup time
  #################################################

  #############################################
  def __init__(self):
    self.identifier = "aarch64-xros"
    self.architecture = "aarch64"
    self.os = "xros"
    self.c_compiler = "clang"
    self.cxx_compiler = "clang++"
    self.supports_host = ["x86_64-macos","aarch64-macos"]
  #############################################
  @property
  def sdkdir(self):
    return obt.path.Path(os.environ["OBT_XROS_SDK_DIR"])
  @property
  def _sdkdir(self):
    IOS_SDK = _xrossdk_private()
    return IOS_SDK._xrosdkstr
  @property
  def _sdkver(self):
    IOS_SDK = _xrossdk_private()
    return IOS_SDK._xrosdkver
  @property
  def _clang_path(self):
    IOS_SDK = _xrossdk_private()
    return IOS_SDK._clang_path
  @property
  def _clangpp_path(self):
    IOS_SDK = _xrossdk_private()
    return IOS_SDK._clang_path
  #############################################
  def misc(self):
    return {
      "sdkdir": self.sdkdir,
      "_sdkdir": self._sdkdir,
      "_sdkver": self._sdkver,
      "_clang_path": self._clang_path,
      "_clangpp_path": self._clangpp_path
    }




###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt import dep, command, path, host


###############################################################################

class fltk(dep.StdProvider):
  #############################################
  name = "fltk"
  def __init__(self):
    super().__init__(fltk.name)
    self.declareDep("cmake")
    self._builder = self.createBuilder(dep.CMakeBuilder)
    self._builder.setCmVar("OPTION_BUILD_SHARED_LIBS","ON")
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=fltk.name,
                             repospec="fltk/fltk",
                             revision="release-1.3.5",
                             recursive=False)
  ########################################################################
  def install(self):
    ok = self._builder.install(self.build_dest)
    if ok:
      if host.IsGentoo:
        ok = (0==command.system(["rm",path.stage()/"lib64"/"libfltk*.a"]))
      else:
        ok = (0==command.system(["rm",path.libs()/"libfltk*.a"]))
    return ok
  #############################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libfltk_SHARED.so").exists()



#!/usr/bin/env python3

from ork import command, path

builddir = path.builds()/"amigadev-test1"

command.run(["docker",
             "run",
              "--mount",
              "type=bind,source=%s,target=/home/amigadev/test1/.build"%builddir,
             "sebastianbergmann/amitools:latest",
              "vamos", 
              "/home/amigadev/test1/.build/main.exe"
              ])



#!/usr/bin/env python3
###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, sys, string, argparse
import obt.search
import obt.path
import obt.dep

parser = argparse.ArgumentParser(description='build all box products')
parser.add_argument('--dep', help='dep to search' )
parser.add_argument('keywords', metavar='K', type=str, nargs='+', help='search keywords')

_args = vars(parser.parse_args())

#################################################################################

if _args["dep"]!=None:
  depname = _args["dep"]
  path_list = [obt.path.builds()/depname]
  ########################
  depnode = obt.dep.DepNode.FIND(depname)
  depinst = depnode.instance
  ########################
  # allow dep module to override default search path
  ########################
  if hasattr(depinst,"find_paths"):
    path_list = depinst.find_paths()
  ########################
  words = _args["keywords"]
  obt.search.execute_at(words,path_list)



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
import os, inspect, tarfile, sys
from pathlib import Path
import importlib.util
import obt.path, obt.host
from obt.command import Command, run
from obt.deco import Deco
from obt.wget import wget
from obt import pathtools, cmake, make, path, git, host, _globals, log, buildtrace
import obt._dep_provider
import obt._dep_enumerate
#import obt._dep_x

deco = Deco()
def module_of_class(clazz):
   return sys.modules[clazz.__module__].__file__

###############################################################################
class DepNode:
    """dependency provider node"""
    def __init__(self,dep_name=None,dep_path=None,with_overrides=True):
      assert(isinstance(dep_name,str))
      self.miscoptions = _globals.getOptions()
      self.name = dep_name
      self.module_path = dep_path
      self.modules_base = Path(obt.path.obt_modules_base()).resolve()
      self.scrname = ("%s.py"%dep_name)
      self.module_spec = importlib.util.spec_from_file_location(self.name, str(self.module_path))
      self.module = importlib.util.module_from_spec(self.module_spec)
      self.module_spec.loader.exec_module(self.module)
      if(hasattr(self.module,dep_name)):
        assert(hasattr(self.module,dep_name))
        self.module_class = getattr(self.module,dep_name)
        #print(self.module_class)
        assert(inspect.isclass(self.module_class))
        assert(issubclass(self.module_class,obt._dep_provider.Provider))
        ##########################################################
        # check for override class in <staging>/"dep-overrides"
        ##########################################################
        override_path = path.stage()/"dep-overrides"/self.scrname
        if with_overrides and override_path.exists():
          self.module_path = override_path
          self.module_spec = importlib.util.spec_from_file_location(self.name, str(self.module_path))
          self.module = importlib.util.module_from_spec(self.module_spec)
          self.module_spec.loader.exec_module(self.module)
          if(hasattr(self.module,dep_name)):
            override_class = getattr(self.module,dep_name)
            assert(inspect.isclass(override_class))
            #print(module_of_class(self.module_class))
            #print(module_of_class(override_class))
            #assert(issubclass(override_class,self.module_class))
            self.module_class = override_class
        ##########################################################
        #print(dep_name)
        self.instance = self.module_class()
        self.instance._node = self
    ## string descriptor of dependency

    def __str__(self):
      return str(self.instance) if hasattr(self,"instance") else "???"

    ## provider method

    def provide(self):
      #print(self,self.name,self.module_path,self.module,self.module_class)
      #print(self.instance)
      if self.instance.exists:
        provide = self.instance.provide()
        assert(provide==True)
        return provide
      else:
        provide = self.instance.provide()
        return provide

    @classmethod
    def nodeForName(cls,named,with_overrides=True):
      n = _globals.getNode(named)
      if n==None:
        n = cls.FIND(named,with_overrides)
        if n!=None:
          _globals.setNode(named,n)
      return n

    @classmethod
    def ALL(cls,with_overrides=True):
      ALL_DEPS = dict()
      e = obt._dep_enumerate._enumerate()
      for key in e.keys():
        val = e[key]
        n = DepNode(key,val._fullpath,with_overrides)
        if n and hasattr(n,"instance") and n.instance.supports_host:
          ALL_DEPS[key]=n
      return ALL_DEPS

    @classmethod
    def FIND(cls,named,with_overrides=True):
      e = obt._dep_enumerate._enumerate()
      n = None
      if named in e.keys():
        val = e[named]
        n = DepNode(named,val._fullpath,with_overrides)
      if n and hasattr(n,"instance") and n.instance.supports_host:
        return n
      return None

###############################################################################
    @classmethod
    def FindWithMethod(cls,named):
      e = obt._dep_enumerate._enumerate()
      rval = {}
      for k in e.keys():
        val = e[k]
        #print(val._fullpath)
        n = DepNode(k,val._fullpath)
        if n and hasattr(n,"instance") and n.instance.supports_host:
          if hasattr(n.instance,named):
            rval[k] = n.instance
      return rval




###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os,io,sys, platform, subprocess, threading
import shlex, errno, pty, select, signal, time

from obt.deco import Deco
from obt import log, pathtools, buildtrace

import obt.path 
deco = Deco()

###############################################################################

def procargs(command_list):
    rval = list()
    if isinstance(command_list, str):
        rval = shlex.split(command_list)
    elif isinstance(command_list,list):
        newlist = []
        for item in command_list:
            newlist.append(str(item))
        rval = newlist
    return rval

###############################################################################

class Command:

    """run a command with a provided environment"""

    ###########################################################################

    def __init__(self, 
                 command_list, 
                 environment=dict(),
                 do_log=True,
                 working_dir=None,
                 use_shell=False):
        #print(command_list)
        assert(type(command_list)==list)
        self.env = os.environ
        self.working_dir = working_dir
        for k in environment.keys():
            self.env[k]=str(environment[k])
        self.command_list = procargs(command_list)
        #print(self.command_list)
        self._do_log = do_log
        self._use_shell = use_shell

    ###########################################################################

    def exec(self,use_shell=False):

        cur_dir = obt.path.Path(os.getcwd())
        
        if self.working_dir!=None:
          pathtools.chdir(self.working_dir)
        else:
          self.working_dir = cur_dir

        if self._do_log:
          log.output("cmdexec: %s"%deco.bright(self.command_list))

        buildtrace.buildTrace({  
         "op": "command(cmd.exec)",
         "working_dir": self.working_dir,
         "arglist": self.command_list, 
         "os_env": dict(self.env), 
         "use_shell": use_shell or self._use_shell })

        child_process = subprocess.Popen( self.command_list,
                                          universal_newlines=True,
                                          env=self.env,
                                          shell=use_shell or self._use_shell )
        child_process.communicate()
        child_process.wait()

        if self.working_dir!=None:
          pathtools.chdir(cur_dir)

        return child_process.returncode


    def exec_filtered(self,use_shell=False,on_line=None):
        assert(on_line!=None)

        buildtrace.buildTrace({  
         "op": "command(cmd.exec_filtered)",
         "curwd": os.getcwd(),
         "arglist": self.command_list, 
         "os_env": dict(self.env), 
         "use_shell": use_shell or self._use_shell })

        def output_reader(child_process):
          for line in iter(child_process.stdout.readline, b''):
            on_line(format(line.decode('utf-8')))

        if self._do_log:
          log.output("cmdexecf %s"%deco.bright(self.command_list))

        child_process = subprocess.Popen(self.command_list,
                                         universal_newlines=False,
                                         shell=use_shell,
                                         stdout=subprocess.PIPE,
                                         stderr=subprocess.STDOUT)

        thr = threading.Thread(target=output_reader,
                               args=(child_process,))

        try:
          thr.start()
          time.sleep(0.25)
          keep_going = True
          while keep_going:
            time.sleep(0.25)
            keep_going = (child_process.poll() == None)
          child_process.stdout.close()
        finally:
          child_process.terminate()
          try:
             child_process.wait(timeout=0.2)
          except subprocess.TimeoutExpired:
             assert(False)
        thr.join()
        return child_process.returncode

    ###########################################################################

    def capture(self):

        buildtrace.buildTrace({  
         "op": "command(cmd.capture)",
         "curwd": os.getcwd(),
         "arglist": self.command_list, 
         "os_env": dict(self.env), 
         "use_shell": False })

        return subprocess.check_output(self.command_list,
                                       universal_newlines=True,
                                       env=self.env )

    ###########################################################################

    def execr(self):

        os.execve(self.command_list[0],self.command_list[1:],self.env)


###############################################################################

def run(command_list, 
        environment=dict(),
        working_dir=None,
        do_log=False):
  assert(type(command_list)==list)
  return Command(command_list,environment,do_log=do_log,working_dir=working_dir).exec()

###############################################################################

def deferredRun(command_list, 
                environment=dict(),
                working_dir=None,
                do_log=False):
  assert(type(command_list)==list)
  return lambda: run(command_list,
                     environment=environment,
                     working_dir=working_dir,
                     do_log=do_log)

###############################################################################

def run_filtered(command_list, environment=dict(),on_line=None,do_log=False):
  return Command(command_list,environment,do_log=do_log).exec_filtered(on_line=on_line)
###############################################################################

def capture(command_list,environment=dict(),do_log=True):
  return Command(command_list,environment,do_log=do_log).capture()

###############################################################################
# Command Chain
#  often you need a list of commands to be executed with exit codes tested
#  if any command in the chain fails (returns non zero exit code)
#   then the chain is terminated (subsequent commands will not be executed)
#  Use a command chain when you dont want to look at a bunch of nested conditional
#   command.run blocks
###############################################################################

class chain:
  def __init__(self):
    self._rval = 0
  def run(self,cmdlist): # conditional run (if all previous succeeded, then run next)
    if self._rval == 0:
      self._rval = run(cmdlist)
    return self._rval
  def ok(self): # have all commands run succeeded so far ?
    return self._rval==0


class chain2:
  def __init__(self,do_log=False):
    self._rval = 0
    self._list = list()
    self._do_log = do_log
  def add(self,cmd_or_list,working_dir=None): 
    if(callable(cmd_or_list)):
      self._list += [cmd_or_list]
    else:
      assert(type(cmd_or_list)==list)
      defcmd = deferredRun(cmd_or_list,working_dir=working_dir,do_log=self._do_log)
      self._list += [defcmd]
  def execute(self): 
    for item in self._list:
      assert(callable(item))
      self._rval = item()
      if self._rval!=0:
        return self._rval 
    return 0
  def ok(self): # have all commands run succeeded so far ?
    return self._rval==0

###############################################################################

def system(command_list,working_dir=None,do_log=False):
  buildtrace.buildTrace({  
   "op": "command(system)",
   "curwd": os.getcwd(),
   "arglist": command_list, 
   "os_env": dict(os.environ) })
  args = procargs(command_list)
  joined = " ".join(args)
  if working_dir!=None:
    os.chdir(str(working_dir))
  if do_log:
    log.output("cmdsys: [%s]"%deco.bright(joined))
  return os.system(joined)

###############################################################################

def subshell(directory=None,prompt=None,environment=dict()):
    cur_dir = os.getcwd()
    os.chdir(str(directory))
    curprompt = os.environ["PS1"]
    bdeco = Deco(bash=True)
    PROMPT = bdeco.promptL('[ <<Dep.Build ]')
    PROMPT += bdeco.promptC("\\w")
    PROMPT += bdeco.promptR("[ %s ]"%prompt)
    PROMPT += bdeco.bright("> ")
    os.environ["PS1"] = PROMPT
    retc = Command(["bash","--norc"],environment=environment).exec()
    os.chdir(cur_dir)
    os.environ["PS1"] = curprompt
    return retc

###############################################################################

cmd = Command 

class factory:
  def __init__(self,prefix=[],wdir=None,do_log=True):
    self._working_dir = wdir 
    self._do_log = do_log
    self._clprefix = prefix
  def cmd(self,*args,extra_args=[]):
    return Command( self._clprefix + list(args) + extra_args, #
                    working_dir=self._working_dir, #
                    do_log=self._do_log )

__all__ =   [ "Command", "cmd","factory" ]



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt import dep, host, path
from obt.command import Command

###############################################################################

class clang(dep.StdProvider):
  def __init__(self):
    name = "clang"
    super().__init__(name=name)
    #self._archlist = ["x86_64"]
    self.llvm = dep.instance("llvm")
    self.hostdesc = host.description()
    if hasattr(self.llvm,"_fetcher"):
      self._builder = self.createBuilder(dep.CMakeBuilder)
      self._builder.requires([self.llvm])
      ##########################################
      # llvm cmake file is 1 subdir deeper than usual
      ##########################################
      self.source_root = path.builds()/"llvm"
      self.build_src = self.source_root/"clang"
      self.build_dest = self.source_root/".build"
  ##########################################
  def __str__(self):
    return "Clang(From LLVM)"
  ##########################################
  @property
  def _fetcher(self):
    fetcher = dep.NopFetcher(clang.name)
    fetcher._revision = self.llvm._fetcher._revision
    return fetcher
  ##########################################
  @property
  def linux_bindir(self):
    if host.IsLinux:
      if host.IsGentoo:
        return path.Path("/usr/lib/llvm/11/bin")
      elif host.IsDebian:
        return path.Path("/usr/bin")
    return path.Path("")
  ##########################################
  @property
  def bin_clangpp(self):
   ub_release = {
      "bionic": "clang++-10",
      "buster": "clang++-13",
      "focal": "clang++-10",
      "jammy": "clang++-12",
      "mantic": "clang++-17",
   }
   if host.IsLinux:
      return self.linux_bindir/ub_release[self.hostdesc.codename]
   return path.Path("clang++")
  ##########################################
  @property
  def bin_clang(self):
   ub_release = {
      "bionic": "clang-10",
      "buster": "clang-11",
      "focal": "clang-10",
      "jammy": "clang-12",
      "mantic": "clang-17",
   }   
   if host.IsLinux:
     return self.linux_bindir/ub_release[self.hostdesc.codename]
   return path.Path("clang")
  ##########################################



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt import dep, host, path, env, log
from obt.path import Path
from obt.command import Command
from obt.deco import Deco
from yarl import URL
deco = Deco()

###############################################################################

class _ispc_from_source(dep.StdProvider):
  name = "ispc"
  def __init__(self):
    super().__init__(_ispc_from_source.name)
    self._archlist = ["x86_64"]
    self.llvm = dep.instance("llvm")
    self._builder = self.createBuilder(dep.CMakeBuilder)
    self._builder.requires([self.llvm])
    if host.IsOsx:
      self._builder.setCmVar("BISON_EXECUTABLE",path.osx_brewopt()/"bison"/"bin"/"bison")
  ########################################################################
  @property
  def _fetcher(self):
    fetcher = dep.GitFetcher(_ispc_from_source.name)
    fetcher._git_url = "https://github.com/ispc/ispc"
    fetcher._cache=False,
    fetcher._recursive=False
    fetcher._revision = "v1.13.0"
    return fetcher

###############################################################################

class _ispc_from_homebrew(dep.HomebrewProvider):
  name = "ispc"
  def __init__(self):
    super().__init__(_ispc_from_homebrew.name,_ispc_from_homebrew.name)
    self._archlist = ["x86_64"]
  def env_init(self):
    binary = self.brew_prefix()/"bin"/"ispc"
    version = ""
    log.marker("registering ispc SDK (%s)"%version)
    env.set("ISPC",binary)

###############################################################################

class _ispc_from_wget(dep.StdProvider):
  name = "ispc"
  def __init__(self):
    super().__init__(_ispc_from_wget.name)
    self._archlist = ["x86_64"]
    self._version = "v1.13.0"
    basename = "ispc-%s-linux"%self._version
    self.filename = "%s.tar.gz" % basename
    self._builder = dep.BinInstaller(_ispc_from_wget.name)
    self.src_dir = path.builds()/"ispc"/basename
    dst_dir = path.stage()
    self._builder.install_item(source=self.src_dir/"bin"/"ispc", \
                               destination=dst_dir/"bin"/"ispc")
  ########################################################################
  @property
  def _fetcher(self):
    baseurl = URL("https://github.com/ispc/ispc/releases/download")
    fetcher = dep.WgetFetcher(_ispc_from_wget.name)
    fetcher._url = baseurl/self._version/self.filename
    fetcher._fname = self.filename
    fetcher._arctype = "tgz"
    fetcher._md5 = "98fe4a03ef0f39f5ebbae77a03c48e71"
    return fetcher

  ########################################################################
  def env_init(self):
    log.marker("registering ispc(%s) SDK"%self._version)
    env.set("ISPC",path.stage()/"bin"/"ispc")

  def install(self):
    return self._builder.install(None)

  def areRequiredSourceFilesPresent(self):
    return (self.src_dir/"LICENSE.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.bin()/"ispc").exists()

###############################################################################

BASE = dep.switch(linux=_ispc_from_wget,
                  macos=_ispc_from_homebrew)

class ispc(BASE):
  def __init__(self):
    super().__init__()



#!/usr/bin/env python3

import os, sys, argparse
import obt.path
import obt.litex
import obt.vivado
import obt.command

build_dir = obt.path.builds()/"nexvsoc"
this_dir = os.path.dirname(os.path.realpath(__file__))

os.chdir(this_dir)

os.environ["build_dir"] = str(build_dir)

parser = argparse.ArgumentParser(description='nexysv SOCer')

parser.add_argument('--buildgw', action="store_true", help='build gateware' )
parser.add_argument('--uploadgw', action="store_true", help='upload gateware' )
parser.add_argument('--connect', action="store_true", help='connect to soc' )
parser.add_argument('--tty', metavar="tty", help='connect to soc' )
parser.add_argument('--litexinit', action="store_true", help='init litex env (and enter)' )
parser.add_argument('--litexshell', action="store_true", help='enter litex env' )
parser.add_argument('--litexupdate', action="store_true", help='update litex env' )

args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

#####################################
triple = {
    "cpu": "lm32",
    "platform": "nexys_video",
    "target": "base"
}
#####################################
def obtlxcmd(cmd):
    obt.command.run(["obt.litex.env.py",
                     "--cpu",triple["cpu"],
                     "--platform",triple["platform"],
                     "--target",triple["target"],
                     cmd ])
#####################################
if args["litexinit"]:
    obtlxcmd("--init")
elif args ["litexshell"]:
    obtlxcmd("--shell")
elif args ["litexupdate"]:
    obtlxcmd("--update")
#####################################
elif args["buildgw"]:
#####################################
    obt.litex.run( triple=triple,
                   cmdlist=["./nexys_video.py","--output-dir",build_dir] )
#####################################
elif args["uploadgw"]:
#####################################
    obt.vivado.run(["-mode","batch",
                    "-source","bit2svf_nexysv.tcl",
                    "-nojournal",
                    "-nolog"])
    obt.litex.run(triple=triple,
                  cmdlist=[obt.path.litex_env_dir()/"build"/"conda"/"bin"/"openocd",
                   "-f","./nexysv.cfg",
                   "-c","init",
                   "-c","'svf %s/gateware/top.svf'"%str(build_dir),
                   "-c","shutdown"])
#####################################
elif args["connect"]:
#####################################
    assert(args["tty"]!=None)
    obt.litex.run( triple=triple,
                   cmd="litex_term %s"%args["tty"] )



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, inspect, tarfile
from pathlib import Path
import importlib.util
import obt.path, obt.host
from obt.command import Command, run
from obt.deco import Deco
from obt.wget import wget
from obt import pathtools, make, path, git, host, _globals, log
from obt._dep_impl import require
from collections.abc import Callable

deco = Deco()

###############################################################################
# BaseBuilder : builder basic interface
###############################################################################

class BaseBuilder(object):
  def __init__(self,name):
    super().__init__()
    self._name = name
    self._deps = []
    self._debug = False
    self._onPostBuild = None
    self._onPostInstall = None
  def requires(self,deplist):
    """declare that this dep module depends on others"""
    self._deps += deplist
  def build(self,srcdir,blddir,wrkdir,incremental=False):
    """execute build phase"""
    require(self._deps)
  def install(self,blddir):
    """execute install phase"""
    return True

###############################################################################
# NopBuilder : do nothing builder (but still installs child dependencies)
###############################################################################

class NopBuilder(BaseBuilder):
  """Do nothing builder (but still installs child dependencies)"""
  def __init__(self,name):
    super().__init__(name)
  def build(self,srcdir,blddir,wrkdir,incremental=False):
    return True



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt import dep, host, command, path, osrelease

###############################################################################

class pangolin(dep.StdProvider):
  name = "pangolin"
  def __init__(self):
    super().__init__(pangolin.name)
    #self._deps = ["pkgconfig"]
    src_root = self.source_root
    #################################################
    self.declareDep("pkgconfig")
    self.declareDep("cmake")
    self.declareDep("libcurl")
    self.declareDep("pybind11")
    #################################################
    self._builder = self.createBuilder(dep.CMakeBuilder)
    self._builder.setCmVar("BUILD_PANGOLIN_LIBUVC","OFF")
    self._builder.setCmVar("BUILD_PANGOLIN_FFMPEG","OFF")
    desc = osrelease.descriptor()
   
    if desc.version_id == "23.10" and host.IsLinux:
      self._builder.setCmVar("CMAKE_CXX_COMPILER","g++-11")
      self._builder.setCmVar("CMAKE_C_COMPILER","gcc-11")
    #################################################
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=pangolin.name,
                             repospec="stevenlovegrove/pangolin",
                             revision="v0.8",
                             recursive=False)
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libpangolin.so").exists()


###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION ="1.7"

import os, tarfile
from obt import dep, host, path, git, cmake, make
from obt.deco import Deco
from obt.wget import wget
from obt.command import Command
from obt.cmake import context

deco = Deco()

###############################################################################

class astcencoder(dep.Provider):

  def __init__(self,miscoptions=None): ############################################

    super().__init__("astcencoder")

    self.source_root = path.builds()/"astcencoder"
    self.build_dest = path.builds()/"astcencoder"/"Source"

  def __str__(self): ##########################################################

    return "ARM ASTC encoder (github-%s)" % VERSION

  def build(self): ##########################################################

    dep.require("openexr")

    os.system("rm -rf %s"%self.source_root)
    git.Clone("https://github.com/ARM-software/astc-encoder",self.source_root,VERSION)
    os.chdir(self.build_dest)
    cmd = Command(["make","-j",host.NumCores])
    err = cmd.exec()
    if err == 0:
      cmd = Command(["ls","-l"])
      err = cmd.exec()
      if err == 0:
        cmd = Command(["cp","astcenc",path.prefix()/"bin"])
        err = cmd.exec()
        self.manifest.touch()
    return (err==0)
